/*
 * process, Distribution repositories and basic setup for Emmanuel Bigeon projects
 * Copyright (C) 2014-2018 E. Bigeon
 * mailto:emmanuel@bigeon.fr
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.proc.ProcessList.java
 * Created on: May 10, 2017
 */
package fr.bigeon.gclc.process;

import fr.bigeon.gclc.command.Command;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.manager.ConsoleInput;
import fr.bigeon.gclc.manager.ConsoleOutput;

/** A command that will flag a task to stop
 *
 * @author Emmanuel Bigeon */
public final class ProcessClear extends Command {
    /** The taskpool */
    private final TaskPool pool;

    /** @param name the command name
     * @param pool the pool */
    public ProcessClear(final String name, final TaskPool pool) {
        super(name);
        this.pool = pool;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#execute(fr.bigeon.gclc.manager.
     * ConsoleOutput, fr.bigeon.gclc.manager.ConsoleInput,
     * java.lang.String[]) */
    @Override
    public void execute(final ConsoleOutput out, final ConsoleInput in,
                        final String... args) throws CommandRunException {
        for (final String id : pool.getPIDs()) {
            if (!pool.get(id).isRunning()) {
                pool.remove(id);
            }
        }
        return;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#tip() */
    @SuppressWarnings("nls")
    @Override
    public String tip() {
        return "Request a process to stop (softly)";
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usageDetail() */
    @Override
    protected String usageDetail() {
        return null;
    }

}
