package fr.bigeon.gclc.socket;

/*-
 * #%L
 * GCLC Socket
 * %%
 * Copyright (C) 2016 - 2018 Bigeon
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.io.IOException;
import java.net.Socket;
import java.util.Collection;

import fr.bigeon.gclc.ConsoleApplication;
import fr.bigeon.gclc.command.Command;
import fr.bigeon.gclc.command.ExitCommand;
import fr.bigeon.gclc.command.HelpExecutor;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.InvalidCommandName;
import fr.bigeon.gclc.manager.ConsoleInput;
import fr.bigeon.gclc.manager.ConsoleOutput;

/** A test-purpose application
 *
 * @author Emmanuel Bigeon */
public class ConsoleTestApplication {

    /** Exit command */
    public static final String EXIT = "exit"; //$NON-NLS-1$

    /** Create the test application.
     * 
     * @param output the output
     * @param input the input
     * @param manager the manager
     * @return create the application */
    @SuppressWarnings("nls")
    public static ConsoleApplication create(final ConsoleOutput output,
                                            final ConsoleInput input,
                                            final ConnexionManager<Socket> manager) {
        try {
            final ConsoleApplication application = new ConsoleApplication(
                    output, input,
                    "Welcome to the test application. Type help or test.",
                    "See you");
            application.add(new ExitCommand(EXIT, application) {
                /* (non-Javadoc)
                 * @see fr.bigeon.gclc.command.ExitCommand#beforeExit() */
                @Override
                protected void beforeExit() {
                    final Collection<String> coll = manager.getConnected();
                    for (final String string : coll) {
                        manager.disconnect(string);
                    }
                }
            });
            application
                    .add(new HelpExecutor("help", application.root));
            application.add(new Command("test") {

                /* (non-Javadoc)
                 * @see fr.bigeon.gclc.command.ICommand#execute(fr.bigeon.gclc.
                 * manager.ConsoleOutput, fr.bigeon.gclc.manager.ConsoleInput,
                 * java.lang.String[]) */
                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    try {
                        output.println("Test command ran fine");
                    } catch (final IOException e) {
                        throw new CommandRunException("manager closed", e,
                                this);
                    }
                }

                @Override
                public String tip() {
                    return "A test command";
                }

                /* (non-Javadoc)
                 * @see fr.bigeon.gclc.command.Command#usageDetail()
                 */
                @Override
                protected String usageDetail() {
                    // TODO Auto-generated method stub
                    // return null;
                    throw new RuntimeException("Not implemented yet");
                }
            });
            application.add(new Command("long") {

                /* (non-Javadoc)
                 * @see fr.bigeon.gclc.command.ICommand#execute(fr.bigeon.gclc.
                 * manager.ConsoleOutput, fr.bigeon.gclc.manager.ConsoleInput,
                 * java.lang.String[]) */
                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    try {
                        Thread.sleep(2000);
                        output.println("Test command ran fine");
                    } catch (IOException | InterruptedException e) {
                        throw new CommandRunException("manager closed", e,
                                this);
                    }
                }

                @Override
                public String tip() {
                    return "A long run test command";
                }

                /* (non-Javadoc)
                 * @see fr.bigeon.gclc.command.Command#usageDetail()
                 */
                @Override
                protected String usageDetail() {
                    // TODO Auto-generated method stub
                    // return null;
                    throw new RuntimeException("Not implemented yet");
                }
            });
            application.add(
                    new RemoteDisconnectCommand<>("out", manager, true));
            return application;
        } catch (final InvalidCommandName e) {
            e.printStackTrace();
        }
        return null;
    }
}
