/**
 * gclc-socket:fr.bigeon.gclc.socket.SocketConsoleApplicationTest.java
 * Created on: Jun 1, 2016
 */
package fr.bigeon.gclc.socket;

/*-
 * #%L
 * GCLC Socket
 * %%
 * Copyright (C) 2016 - 2018 Bigeon
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import static org.junit.Assert.assertEquals;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.logging.Logger;

import org.junit.Test;

/** Test class for {@link SocketConsoleApplicationShell}
 *
 * @author Emmanuel Bigeon */
@SuppressWarnings({"static-method", "javadoc", "nls"})
public class SocketConsoleApplicationTest {

    private static final Logger LOGGER = Logger
            .getLogger(SocketConsoleApplicationTest.class.getName());

    /** @param in the input
     * @return the string
     * @throws IOException if the input reading failed */
    private String consumeToPrompt(final String server,
                                   final BufferedReader in) throws IOException {
        String fromServer = server;
        LOGGER.fine("Server: \n" + fromServer);
        while (fromServer != null && !fromServer.equals("Bye.") &&
               !fromServer.equals("> ")) {
            fromServer = in.readLine();
            LOGGER.fine("Server: \n" + fromServer);
        }
        return fromServer;
    }

    @Test
    public void testIntegration() throws IOException, InterruptedException {
        Thread server;
        server = TestServer.getServer();
        Thread.sleep(1000);
        final String hostName = "127.0.0.1";
        final int portNumber = 3300;

        try (Socket kkSocket = new Socket(hostName, portNumber);
             PrintWriter out = new PrintWriter(kkSocket.getOutputStream(),
                     true);
             BufferedReader in = new BufferedReader(
                     new InputStreamReader(kkSocket.getInputStream()));) {

            String fromServer;
            int i = -1;
            final String[] cmds = {"help", "toto", "test", "out"};
            while ((fromServer = in.readLine()) != null) {
                i++;
                fromServer = consumeToPrompt(fromServer, in);
                if (fromServer == null || fromServer.equals("Bye.")) {
                    break;
                }

                final String fromUser = cmds[i];
                if (fromUser != null) {
                    LOGGER.fine("Client: " + fromUser);
                    out.println(fromUser);
                }
            }
            assertEquals("Disconnection command should close connection", 3, i);
        }

        try (Socket kkSocket = new Socket(hostName, portNumber);
             PrintWriter out = new PrintWriter(kkSocket.getOutputStream(),
                     true);
             BufferedReader in = new BufferedReader(
                     new InputStreamReader(kkSocket.getInputStream()));) {

            String fromServer;
            int i = 0;
            final String[] cmds = {"help", "toto", "test",
                    ConsoleTestApplication.EXIT};
            while ((fromServer = in.readLine()) != null) {
                fromServer = consumeToPrompt(fromServer, in);
                if (fromServer == null || fromServer.equals("Bye.")) {
                    break;
                }
                LOGGER.info("Server: \n" + fromServer);

                final String fromUser = cmds[i];
                if (fromUser != null) {
                    LOGGER.info("Client: " + fromUser);
                    out.println(fromUser);
                }
                i++;
            }
            assertEquals("Application exit command should close connection", 4,
                    i);
        }
        Thread.sleep(100);
        TestServer.closeServer();
        server.join();
        server = TestServer.getServer();
        Thread.sleep(1000);


        try (Socket kkSocket = new Socket(hostName, portNumber);
             PrintWriter out = new PrintWriter(kkSocket.getOutputStream(),
                     true);
             BufferedReader in = new BufferedReader(
                     new InputStreamReader(kkSocket.getInputStream()));) {

            String fromServer;
            int i = 0;
            final String[] cmds = {"help", "toto", "test",
                    ConsoleTestApplication.EXIT};
            while ((fromServer = in.readLine()) != null) {
                fromServer = consumeToPrompt(fromServer, in);
                if (fromServer == null || fromServer.equals("Bye.")) {
                    break;
                }

                final String fromUser = cmds[i];
                if (fromUser != null) {
                    LOGGER.fine("Client: " + fromUser);
                    out.println(fromUser);
                }
                i++;
            }
            assertEquals("Application exit command should close connection", 4,
                    i);
        }
        TestServer.closeServer();
        server.join();
    }
}
